// -------------------------------------------------
// libhtdigphp.c
//
// Code for libhtdig PHP 4.0 wrapper.
// 
// Requires libhtdig 3.2.0 or better
//
// Part of the ht://Dig package   <http://www.htdig.org/>
// <http://www.gnu.org/copyleft/gpl.html>
//
// Copyright (c) 1995-2002 The ht://Dig Group
// Copyright (c) 2002 Rightnow Technologies, Inc.
// 
// Dual Licensed under GPL & LGPL
// See GPL and LGPL files for License.
// 
// Copies available at 
// http://www.fsf.org/licenses/gpl.html
// http://www.fsf.org/licenses/lgpl.html
//
// --------------------------------------------------

/* $Id: htdigphp.c,v 1.7 2002/03/21 20:34:44 nealr Exp $ */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "htdigphp.h"


/* PHP Includes */
#include "ext/standard/file.h"
#include "ext/standard/info.h"

/* for fileno() */
#include <stdio.h>
#include <time.h>

/* HtDig includes */
#include <libhtdig_api.h>

PHP_FUNCTION(htsearch_open);
PHP_FUNCTION(htsearch_query);
PHP_FUNCTION(htsearch_get_nth_match);
PHP_FUNCTION(htsearch_close);
PHP_FUNCTION(htsearch_get_error);


function_entry htdig_functions[] = 
{
    PHP_FE(htsearch_open, NULL)
    PHP_FE(htsearch_query, NULL)
    PHP_FE(htsearch_get_nth_match, NULL)
    PHP_FE(htsearch_close, NULL) PHP_FE(htsearch_get_error, NULL) {NULL, NULL, NULL}
};

zend_module_entry htdig_module_entry = 
{
    STANDARD_MODULE_HEADER,
    "htdig",
    htdig_functions,
    PHP_MINIT(htdig),
    NULL,
    NULL,
    NULL,
    PHP_MINFO(htdig),
    NO_VERSION_YET,
    STANDARD_MODULE_PROPERTIES
};

#ifdef COMPILE_DL_HTDIG
ZEND_GET_MODULE(htdig)
#endif
//static int le_htdig;

static int num_search_results;

static char time_format[TIME_FORMAT_SIZE];

//static void php_htdig_close(zend_rsrc_list_entry * rsrc TSRMLS_DC);

PHP_MINIT_FUNCTION(htdig)
{
    /* Register the resource, with destructor (arg 1) and text description (arg 3), the 
       other arguments are just standard placeholders */
    //le_htdig = zend_register_list_destructors_ex(php_htdig_close, NULL, "HtDig 3.2.0", module_number);

    return SUCCESS;
}

PHP_MINFO_FUNCTION(htdig)
{
    php_info_print_table_start();
    php_info_print_table_row(2, "HtDig Support", "Enabled");
    php_info_print_table_row(2, "HtDig Version", "3.2.0b4");
    php_info_print_table_end();
}

/* 
 *  htsearch_open
 * 
 *  Wrapper for 
 *  int htsearch_open(htsearch_parameters_struct *);
 *
 *  TODO recieve configFile, debug, logFile as parameters.  2 & 3 are Optional!
 * 
 */
PHP_FUNCTION(htsearch_open)
{
    zval **config_array_arg, **pvalue;
    HashTable *config_array_ht;
    htsearch_parameters_struct htsearch_params;
    int array_size = 0;

    if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &config_array_arg) == FAILURE)
        WRONG_PARAM_COUNT;

    if (Z_TYPE_PP(config_array_arg) != IS_ARRAY)
    {
        php_error(E_WARNING, "First argument to htsearch_open() should be an array");
        return;
    }


    //SEPARATE_ZVAL(config_array_arg);
    convert_to_array_ex(config_array_arg);

    config_array_ht = Z_ARRVAL_PP(config_array_arg);

    array_size = zend_hash_num_elements(config_array_ht);

    if (array_size != 3)
    {
        php_error(E_WARNING, "First argument to htsearch_open() should be an array with 3 elements");
        return;
    }

    if (zend_hash_find(config_array_ht, "configFile", sizeof("configFile"), 
                        (void **) &pvalue) == SUCCESS)
    {
        //SEPARATE_ZVAL(pvalue);
        convert_to_string_ex(pvalue);
        sprintf(htsearch_params.configFile, Z_STRVAL_PP(pvalue));
        //sprintf(htsearch_params.configFile, "/home/httpd/cgi-bin/nealr.db/archive/archive.conf");
    }
    else
    {
        php_error(E_WARNING, "'configFile' key not in array parameter");
        return;
    }


    if (zend_hash_find(config_array_ht, "debug", sizeof("debug"), 
                        (void **) &pvalue) == SUCCESS)
    {
        //SEPARATE_ZVAL(pvalue);
        convert_to_long_ex(pvalue);
        htsearch_params.debug = Z_LVAL_PP(pvalue);
    }
    else
    {
        php_error(E_WARNING, "'debug' key not in array parameter");
        return;
    }

    if (zend_hash_find(config_array_ht, "logFile", sizeof("logFile"), 
                        (void **) &pvalue) == SUCCESS)
    {
        //SEPARATE_ZVAL(pvalue);
        convert_to_string_ex(pvalue);
        sprintf(htsearch_params.logFile, Z_STRVAL_PP(pvalue));
        //sprintf(htsearch_params.configFile, "/home/httpd/cgi-bin/nealr.db/archive/archive.conf");
    }
    else
    {
        php_error(E_WARNING, "'logFile' key not in array parameter");
        return;
    }

    snprintf(time_format, TIME_FORMAT_SIZE, "%s", DEFAULT_TIME_FORMAT);

    htsearch_open(&htsearch_params);

    num_search_results = 0;

    RETURN_LONG(SUCCESS);

}

/* 
 *  wrapper for htsearch_query(htsearch_query_struct *)
 * 
 *  TODO recieve query, algorithms_flag, sortby_flag, format as parameters
 *  Some Optional??
 *
 *       "raw_query" => "$p_query"
 *       "algorithm" => HTSEARCH_ALG_AND_STR
 *       "sortby" => HTSEARCH_SORT_SCORE_STR
 *       "format" => HTSEARCH_FORMAT_LONG_STR
 *       "time_format"                          **** optional *** 
 *
 *
 */
PHP_FUNCTION(htsearch_query)
{
    zval **query_array_arg, **pvalue;
    HashTable *query_array_ht;
    htsearch_query_struct the_query;
    int array_size = 0;

    num_search_results = 0;

    if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &query_array_arg) == FAILURE)
        WRONG_PARAM_COUNT;

    if (Z_TYPE_PP(query_array_arg) != IS_ARRAY)
    {
        php_error(E_WARNING, "First argument to htsearch_query() should be an array");
        return;
    }


    //SEPARATE_ZVAL(query_array_arg);
    convert_to_array_ex(query_array_arg);

    query_array_ht = Z_ARRVAL_PP(query_array_arg);

    array_size = zend_hash_num_elements(query_array_ht);

    if (array_size < 4)
    {
        php_error(E_WARNING, "First argument to htsearch_open() should be an array with at least 4 elements");
        return;
    }

    if (zend_hash_find(query_array_ht, "raw_query", sizeof("raw_query"), 
                        (void **) &pvalue) == SUCCESS)
    {
        //SEPARATE_ZVAL(pvalue);
        convert_to_string_ex(pvalue);
        sprintf(the_query.raw_query, Z_STRVAL_PP(pvalue));
    }
    else
    {
        php_error(E_WARNING, "'raw_query' key not in array parameter");
        return;
    }

    if (zend_hash_find(query_array_ht, "algorithm", sizeof("algorithm"), 
                        (void **) &pvalue) == SUCCESS)
    {
        //SEPARATE_ZVAL(pvalue);
        convert_to_string_ex(pvalue);

        if (strcmp(HTSEARCH_ALG_AND_STR, Z_STRVAL_PP(pvalue)) == 0)
            the_query.algorithms_flag = HTSEARCH_ALG_AND;
        else if (strcmp(HTSEARCH_ALG_OR_STR, Z_STRVAL_PP(pvalue)) == 0)
            the_query.algorithms_flag = HTSEARCH_ALG_OR;
        else if (strcmp(HTSEARCH_ALG_BOOLEAN_STR, Z_STRVAL_PP(pvalue)) == 0)
            the_query.algorithms_flag = HTSEARCH_ALG_BOOLEAN;
        else
        {
            php_error(E_WARNING, "'algorithm' value not valid [%s]", Z_STRVAL_PP(pvalue));
            return;
        }
    }
    else
    {
        php_error(E_WARNING, "'algorithm' key not in array parameter");
        return;
    }

    if (zend_hash_find(query_array_ht, "format", sizeof("format"), 
                        (void **) &pvalue) == SUCCESS)
    {
        //SEPARATE_ZVAL(pvalue);
        convert_to_string_ex(pvalue);

        if (strcmp(HTSEARCH_FORMAT_LONG_STR, Z_STRVAL_PP(pvalue)) == 0)
            the_query.format = HTSEARCH_FORMAT_LONG;
        else if (strcmp(HTSEARCH_FORMAT_SHORT_STR, Z_STRVAL_PP(pvalue)) == 0)
            the_query.format = HTSEARCH_FORMAT_SHORT;
        else
        {
            php_error(E_WARNING, "'format' value not valid [%s]", Z_STRVAL_PP(pvalue));
            return;
        }

    }
    else
    {
        php_error(E_WARNING, "'format' key not in array parameter");
        return;
    }

    if (zend_hash_find(query_array_ht, "sortby", sizeof("sortby"), 
                        (void **) &pvalue) == SUCCESS)
    {
        //SEPARATE_ZVAL(pvalue);
        convert_to_string_ex(pvalue);

        if (strcmp(HTSEARCH_SORT_SCORE_STR, Z_STRVAL_PP(pvalue)) == 0)
            the_query.sortby_flag = HTSEARCH_SORT_SCORE;
        else if (strcmp(HTSEARCH_SORT_REV_SCORE_STR, Z_STRVAL_PP(pvalue)) == 0)
            the_query.sortby_flag = HTSEARCH_SORT_REV_SCORE;
        else if (strcmp(HTSEARCH_SORT_TIME_STR, Z_STRVAL_PP(pvalue)) == 0)
            the_query.sortby_flag = HTSEARCH_SORT_TIME;
        else if (strcmp(HTSEARCH_SORT_REV_TIME_STR, Z_STRVAL_PP(pvalue)) == 0)
            the_query.sortby_flag = HTSEARCH_SORT_REV_TIME;
        else if (strcmp(HTSEARCH_SORT_TITLE_STR, Z_STRVAL_PP(pvalue)) == 0)
            the_query.sortby_flag = HTSEARCH_SORT_TITLE;
        else if (strcmp(HTSEARCH_SORT_REV_TITLE_STR, Z_STRVAL_PP(pvalue)) == 0)
            the_query.sortby_flag = HTSEARCH_SORT_REV_TITLE;
        else
        {
            php_error(E_WARNING, "'sortby' value not valid [%s]", Z_STRVAL_PP(pvalue));
            return;
        }

    }
    else
    {
        php_error(E_WARNING, "'sortby' key not in array parameter");
        return;
    }

    if (zend_hash_find(query_array_ht, "time_format", sizeof("time_format"), 
                        (void **) &pvalue) == SUCCESS)
    {
        //SEPARATE_ZVAL(pvalue);
        convert_to_string_ex(pvalue);

        snprintf(time_format, TIME_FORMAT_SIZE, "%s", Z_STRVAL_PP(pvalue));
    }

    
    num_search_results = htsearch_query(&the_query);

    //printf("[%s][%d]\n", the_query.raw_query, num_results);

    RETURN_LONG(num_search_results);
    
}

/* 
 *
 * 
 */
PHP_FUNCTION(htsearch_get_nth_match)
{
    zval **result_num_arg, **pvalue;
    htsearch_query_match_struct result;
    int result_num;
    char local_time_str[TIME_FORMAT_SIZE];

    if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &result_num_arg) == FAILURE)
        WRONG_PARAM_COUNT;

    if (Z_TYPE_PP(result_num_arg) != IS_LONG)
    {
        php_error(E_WARNING, "First argument to htsearch_get_nth_match() should be an integer");
        return;
    }

    convert_to_long_ex(result_num_arg);
    result_num = Z_LVAL_PP(result_num_arg);

    if (array_init(return_value) == FAILURE) {
        RETURN_FALSE;
    }
    
    htsearch_get_nth_match(result_num, &result);

    strftime(local_time_str, TIME_FORMAT_SIZE, time_format, &result.time_tm);
    //fprintf(stderr, "time:[%s]\n", asctime(&result.time_tm));
    
    add_assoc_string(return_value, "title",          result.title,         1);
    add_assoc_string(return_value, "URL",            result.URL,           1);
    add_assoc_string(return_value, "excerpt",        result.excerpt,       1);
    add_assoc_string(return_value, "time",           local_time_str,       1);
    add_assoc_long(  return_value, "score",          result.score           );
    add_assoc_long(  return_value, "scorepercent",   result.score_percent   );
    add_assoc_long(  return_value, "size",           result.size            );
    add_assoc_long(  return_value, "size",           result.size            );

    //fprintf(stderr, "%s\n", result.title);
    
}

/* 
 * 
 *
 */
PHP_FUNCTION(htsearch_close)
{
    int ret = 0;

    ret = htsearch_close();

    RETURN_LONG(ret);
}

/* 
 * 
 *
 */
PHP_FUNCTION(htsearch_get_error)
{
    RETURN_LONG(SUCCESS);
}

/*
static void php_htdig_close(zend_rsrc_list_entry * rsrc TSRMLS_DC)
{    

}

/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: sw=4 ts=4 tw=78 fdm=marker
 * vim<600: sw=4 ts=4 tw=78
 */
